from crypto import PasswordHasher
from model import Session, User
from exceptions import InvalidCredentialsError, UserExistsError


class UserOps:
    def __init__(self, username, password):
        self.username = username
        self.password = password

    def authenticate(self):
        """
        Attempts to authenticate the user.
        Raises an exception on failure, does nothing on success.

        :raises InvalidCredentialsError:
            User does not exist or password provided is invalid
        """
        with Session() as db:

            user = db.query(User).filter(User.username == self.username).first()

            if not user or not PasswordHasher.verify(self.password, user.passwordhash):
                raise InvalidCredentialsError

    def register(self):
        """
        Attempts to register a user.
        Raises an exception of failure, does nothing on success.

        :raises UserExistsError:
            A user with the provided username already exists
        """
        with Session() as db:

            if db.query(User).filter(User.username == self.username).all():
                raise UserExistsError

            user = User(username=self.username,
                        passwordhash=PasswordHasher.hash(self.password))
            db.add(user)
            db.commit()
